@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP #-}
+
 -- |
 -- Module: Data.Enumerator.Text
 -- Copyright: 2010-2011 John Millikin
@@ -721,7 +723,7 @@ isolateWhile p (Continue k) = continue loop where
 	loop (Chunks []) = continue loop
 	loop (Chunks xs) = iter where
 		lazy = TL.fromChunks xs
-		(s1, s2) = TL.span p lazy
+		(s1, s2) = tlSpanBy p lazy
 		iter = if TL.null s2
 			then k (Chunks xs) >>== isolateWhile p
 			else k (toChunks s1) >>== (`yield` toChunks s2)
@@ -751,32 +753,65 @@ splitWhen p = loop where
 lines :: Monad m => Enumeratee T.Text T.Text m b
 lines = splitWhen (== '\n')
 
--- | Read lines of text from the handle, and stream them to an 'Iteratee'.
+-- | Read lines of text from a handle, and stream them to an 'Iteratee'.
 -- If an exception occurs during file IO, enumeration will stop and 'Error'
 -- will be returned. Exceptions from the iteratee are not caught.
 --
 -- The handle should be opened with an appropriate text encoding, and
 -- in 'IO.ReadMode' or 'IO.ReadWriteMode'.
 --
+-- This function may be significantly slower than using
+-- @Data.Enumerator.Binary.enumHandle@, due to the additional overhead of
+-- decoding input data to Unicode. Users who can depend on their input files
+-- being in a certain encoding (such as UTF8) are encouraged to use binary
+-- input and 'decode'.
+--
+-- Changed in 0.4.18: Lines streamed from 'enumHandle' and 'enumFile' now
+-- include their trailing newline.
+--
 -- Since: 0.2
 enumHandle :: MonadIO m => IO.Handle
            -> Enumerator T.Text m b
 enumHandle h = checkContinue0 $ \loop k -> do
-	let getText = Exc.catch
-		(Just `fmap` TIO.hGetLine h)
-		(\err -> if isEOFError err
-			then return Nothing
-			else Exc.throwIO err)
-	
-	maybeText <- tryIO getText
+	maybeText <- tryIO (textGetLine h)
 	case maybeText of
 		Nothing -> continue k
 		Just text -> k (Chunks [text]) >>== loop
-	
 
+textGetLine :: IO.Handle -> IO (Maybe T.Text)
+textGetLine h = loop [] where
+#if MIN_VERSION_base(4,2,0)
+	pack = T.pack
+#else
+	pack = TE.decodeUtf8 . B8.pack
+#endif
+	loop acc = Exc.catch
+		(do
+			c <- IO.hGetChar h
+			if c == '\n'
+				then return (Just (pack (reverse (c:acc))))
+				else loop (c:acc))
+		(\err -> if isEOFError err
+			then case acc of
+				[] -> return Nothing
+				_ -> return (Just (pack (reverse acc)))
+			else Exc.throwIO err)
 
--- | Opens a file path in text mode, and passes the handle to 'enumHandle'.
--- The file will be closed when the 'Iteratee' finishes.
+-- | Read lines of text from a file, and stream them to an 'Iteratee'.
+-- If an exception occurs during file IO, enumeration will stop and 'Error'
+-- will be returned. Exceptions from the iteratee are not caught.
+--
+-- The file will be opened in text mode, and will be closed when the
+-- 'Iteratee' finishes.
+--
+-- This function may be significantly slower than using
+-- @Data.Enumerator.Binary.enumFile@, due to the additional overhead of
+-- decoding input data to Unicode. Users who can depend on their input files
+-- being in a certain encoding (such as UTF8) are encouraged to use binary
+-- input and 'decode'.
+--
+-- Changed in 0.4.18: Lines streamed from 'enumHandle' and 'enumFile' now
+-- include their trailing newline.
 --
 -- Since: 0.2
 enumFile :: FilePath -> Enumerator T.Text IO b
